﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Windows.Controls;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace WmkRenderLayer
{
    public class ExtendedWpfMap : WpfMap
    {       

        /// <summary>
        /// Occurs when drawing progress changed.
        /// </summary>
        public event EventHandler<DrawingProgressChangedExtendedWpfMapEventArgs> DrawingProgressChanged;

        public ExtendedWpfMap()
            : base()
        {
            Overlays.CollectionChanged += Overlays_CollectionChanged;
            this.ComputeCurrentProgressUsingBuffer = false;
        }

        public bool ComputeCurrentProgressUsingBuffer { get; set; }

        protected virtual void OnDrawingProgressChanged(DrawingProgressChangedExtendedWpfMapEventArgs e)
        {            
            if (DrawingProgressChanged != null)
            {
                DrawingProgressChanged(this, e);
            }
        }

        private void Overlays_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (Overlay overlay in e.NewItems)
                    {
                        TileOverlay tileOverlay = overlay as TileOverlay;
                        if (tileOverlay != null && tileOverlay.TileType != TileType.SingleTile)
                        {
                            tileOverlay.DrawTilesProgressChanged += tileOverlay_DrawTilesProgressChanged;
                            tileOverlay.Drawing += new EventHandler<DrawingOverlayEventArgs>(tileOverlay_Drawing);
                            tileOverlay.Drawn +=new EventHandler<DrawnOverlayEventArgs>(tileOverlay_Drawn);
                        }
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (Overlay overlay in e.NewItems)
                    {
                        TileOverlay tileOverlay = overlay as TileOverlay;
                        if (tileOverlay != null && tileOverlay.TileType != TileType.SingleTile)
                        {
                            tileOverlay.DrawTilesProgressChanged -= tileOverlay_DrawTilesProgressChanged;
                        }
                    }
                    break;
                default:
                    break;
            }
        }

        System.Diagnostics.Stopwatch watch = new System.Diagnostics.Stopwatch();
        void tileOverlay_Drawing(object sender, DrawingOverlayEventArgs e)
        {
            watch.Start();
        }

        void tileOverlay_Drawn(object sender, DrawnOverlayEventArgs e)
        {
            watch.Stop();

            Console.WriteLine(string.Format("total time to draw tile = {0}", watch.ElapsedMilliseconds));
            watch.Reset();
        }

        int previousPercentage = 0;
        private void tileOverlay_DrawTilesProgressChanged(object sender, DrawTilesProgressChangedTileOverlayEventArgs e)
        {
            TileOverlay currentTileOverlay = (TileOverlay)sender;
            currentTileOverlay.OverlayCanvas.Tag = e.ProgressPercentage;            

            IEnumerable<TileOverlay> tileOverlays = Overlays.OfType<TileOverlay>()
                .Where((to => to.TileType != TileType.SingleTile))
                .Where(to => to.OverlayCanvas.Tag != null && (int)to.OverlayCanvas.Tag != 100);

            if (ComputeCurrentProgressUsingBuffer)
                ComputeCurrentDrawProgressUsingBuffer(tileOverlays);
            else
                ComputeCurrenDrawProgressWithoutBuffer(tileOverlays);
        }

        private void ComputeCurrenDrawProgressWithoutBuffer(IEnumerable<TileOverlay> tileOverlays)
        {
            var tilesInVisibleExtent = tileOverlays.SelectMany(o => ((Canvas)o.OverlayCanvas.Children[0]).Children
                                                           .OfType<ThinkGeo.MapSuite.WpfDesktopEdition.Tile>()
                                                           .Where(tile => o.MapArguments.CurrentExtent.Contains(tile.TargetExtent)));

            var totalTiles = tileOverlays.SelectMany(o => ((Canvas)o.OverlayCanvas.Children[0]).Children
                                                           .OfType<ThinkGeo.MapSuite.WpfDesktopEdition.Tile>());
            int total = tilesInVisibleExtent.Count();
            int currentProgress = 100;

            if (total != 0)
            {
                var tilesLoaded = tilesInVisibleExtent.Where(tile => tile.IsOpened);
                currentProgress = tilesLoaded.Count() * 100 / total;
            }

            if (Math.Abs(currentProgress - previousPercentage) >= 5 || currentProgress == 100 && previousPercentage != 100)
            {
                previousPercentage = currentProgress;
                Console.WriteLine("{0}% tiles in visible extent={1}, all tiles={2}", currentProgress, total, totalTiles.Count());
                OnDrawingProgressChanged(new DrawingProgressChangedExtendedWpfMapEventArgs(currentProgress, null));
            }
        }

        private void ComputeCurrentDrawProgressUsingBuffer(IEnumerable<TileOverlay> tileOverlays)
        {
            int total = tileOverlays.Count();
            int currentProgress = 100;
            if (total != 0)
            {
                int progress = 0;
                foreach (TileOverlay tileOverlay in tileOverlays)
                {
                    if (tileOverlay.OverlayCanvas.Tag != null)
                    {
                        var overlayProgress = (int)tileOverlay.OverlayCanvas.Tag;
                        progress += overlayProgress;
                    }
                }

                currentProgress = progress / total;

                if (Math.Abs(currentProgress - previousPercentage) >= 5 || currentProgress == 100 && previousPercentage != 100)
                {
                    previousPercentage = currentProgress;
                    Console.WriteLine("{0}%", currentProgress);
                    OnDrawingProgressChanged(new DrawingProgressChangedExtendedWpfMapEventArgs(currentProgress, null));
                }
            }
        }
    }
}
